package mx.managers
{
   import flash.display.DisplayObject;
   import flash.events.Event;
   import flash.events.EventDispatcher;
   import flash.events.MouseEvent;
   import flash.events.TimerEvent;
   import flash.geom.Point;
   import flash.geom.Rectangle;
   import flash.utils.Timer;
   import mx.controls.ToolTip;
   import mx.core.Application;
   import mx.core.IInvalidating;
   import mx.core.IToolTip;
   import mx.core.IUIComponent;
   import mx.core.mx_internal;
   import mx.effects.Effect;
   import mx.effects.EffectManager;
   import mx.events.EffectEvent;
   import mx.events.ToolTipEvent;
   import mx.styles.IStyleClient;
   import mx.validators.IValidatorListener;
   
   use namespace mx_internal;
   
   public class ToolTipManagerImpl extends EventDispatcher implements IToolTipManager
   {
      private static var instance:IToolTipManager;
      
      mx_internal static const VERSION:String = "2.0.1.0";
      
      private var _enabled:Boolean = true;
      
      private var _showDelay:Number = 500;
      
      private var _hideEffect:Effect;
      
      private var hideTimer:Timer;
      
      private var _toolTipClass:Class;
      
      private var _scrubDelay:Number = 100;
      
      private var showTimer:Timer;
      
      private var _currentToolTip:IToolTip;
      
      private var currentText:String;
      
      private var scrubTimer:Timer;
      
      private var previousTarget:DisplayObject;
      
      private var _currentTarget:DisplayObject;
      
      private var _showEffect:Effect;
      
      private var initialized:Boolean = false;
      
      private var _hideDelay:Number = 10000;
      
      private var isError:Boolean;
      
      public function ToolTipManagerImpl()
      {
         initialized = false;
         _enabled = true;
         _hideDelay = 10000;
         _scrubDelay = 100;
         _showDelay = 500;
         _toolTipClass = ToolTip;
         super();
         if(instance)
         {
            throw new Error("Instance already exists.");
         }
      }
      
      public static function getInstance() : IToolTipManager
      {
         if(!instance)
         {
            instance = new ToolTipManagerImpl();
         }
         return instance;
      }
      
      private function toolTipMouseOverHandler(param1:MouseEvent) : void
      {
         checkIfTargetChanged(DisplayObject(param1.target));
      }
      
      private function showTimer_timerHandler(param1:TimerEvent) : void
      {
         if(currentTarget)
         {
            createTip();
            initializeTip();
            positionTip();
            showTip();
         }
      }
      
      private function systemManager_mouseDownHandler(param1:MouseEvent) : void
      {
         reset();
      }
      
      public function set showDelay(param1:Number) : void
      {
         _showDelay = param1;
      }
      
      private function hideEffectEnded() : void
      {
         var _loc1_:ToolTipEvent = null;
         reset();
         if(previousTarget)
         {
            _loc1_ = new ToolTipEvent(ToolTipEvent.TOOL_TIP_END);
            _loc1_.toolTip = currentToolTip;
            previousTarget.dispatchEvent(_loc1_);
         }
      }
      
      public function set scrubDelay(param1:Number) : void
      {
         _scrubDelay = param1;
      }
      
      public function get currentToolTip() : IToolTip
      {
         return _currentToolTip;
      }
      
      private function toolTipMouseOutHandler(param1:MouseEvent) : void
      {
         checkIfTargetChanged(param1.relatedObject);
      }
      
      public function get enabled() : Boolean
      {
         return _enabled;
      }
      
      public function createToolTip(param1:String, param2:Number, param3:Number, param4:String = null, param5:IUIComponent = null) : IToolTip
      {
         var _loc6_:ToolTip = null;
         var _loc7_:ISystemManager = null;
         _loc6_ = new ToolTip();
         _loc7_ = !!param5 ? param5.systemManager : Application.application.systemManager;
         _loc7_.toolTipChildren.addChild(_loc6_);
         if(param4)
         {
            _loc6_.setStyle("styleName","errorTip");
            _loc6_.setStyle("borderStyle",param4);
         }
         _loc6_.text = param1;
         sizeTip(_loc6_);
         _loc6_.move(param2,param3);
         return _loc6_ as IToolTip;
      }
      
      private function reset() : void
      {
         var _loc1_:ISystemManager = null;
         showTimer.reset();
         hideTimer.reset();
         if(currentToolTip)
         {
            if(Boolean(showEffect) || Boolean(hideEffect))
            {
               currentToolTip.removeEventListener(EffectEvent.EFFECT_END,effectEndHandler);
            }
            EffectManager.endEffectsForTarget(currentToolTip);
            _loc1_ = getSystemManager(previousTarget);
            _loc1_.toolTipChildren.removeChild(DisplayObject(currentToolTip));
            currentToolTip = null;
            scrubTimer.delay = scrubDelay;
            scrubTimer.reset();
            if(scrubDelay > 0)
            {
               scrubTimer.delay = scrubDelay;
               scrubTimer.start();
            }
         }
      }
      
      public function set currentToolTip(param1:IToolTip) : void
      {
         _currentToolTip = param1;
      }
      
      private function showTip() : void
      {
         var _loc1_:ToolTipEvent = null;
         var _loc2_:ISystemManager = null;
         _loc1_ = new ToolTipEvent(ToolTipEvent.TOOL_TIP_SHOW);
         _loc1_.toolTip = currentToolTip;
         currentTarget.dispatchEvent(_loc1_);
         if(isError)
         {
            currentTarget.addEventListener("change",changeHandler);
         }
         else
         {
            _loc2_ = getSystemManager(currentTarget);
            _loc2_.addEventListener(MouseEvent.MOUSE_DOWN,systemManager_mouseDownHandler);
         }
         currentToolTip.visible = true;
         if(!showEffect)
         {
            showEffectEnded();
         }
      }
      
      private function effectEndHandler(param1:EffectEvent) : void
      {
         if(param1.effectInstance.effect == showEffect)
         {
            showEffectEnded();
         }
         else if(param1.effectInstance.effect == hideEffect)
         {
            hideEffectEnded();
         }
      }
      
      public function get hideDelay() : Number
      {
         return _hideDelay;
      }
      
      public function get toolTipClass() : Class
      {
         return _toolTipClass;
      }
      
      public function get currentTarget() : DisplayObject
      {
         return _currentTarget;
      }
      
      public function get hideEffect() : Effect
      {
         return _hideEffect;
      }
      
      private function showEffectEnded() : void
      {
         var _loc1_:ToolTipEvent = null;
         if(hideDelay == 0)
         {
            hideTip();
         }
         else if(hideDelay < Infinity)
         {
            hideTimer.delay = hideDelay;
            hideTimer.start();
         }
         if(currentTarget)
         {
            _loc1_ = new ToolTipEvent(ToolTipEvent.TOOL_TIP_SHOWN);
            _loc1_.toolTip = currentToolTip;
            currentTarget.dispatchEvent(_loc1_);
         }
      }
      
      public function set enabled(param1:Boolean) : void
      {
         _enabled = param1;
      }
      
      private function errorTipMouseOverHandler(param1:MouseEvent) : void
      {
         checkIfTargetChanged(DisplayObject(param1.target));
      }
      
      public function get scrubDelay() : Number
      {
         return _scrubDelay;
      }
      
      public function get showDelay() : Number
      {
         return _showDelay;
      }
      
      public function registerErrorString(param1:DisplayObject, param2:String) : void
      {
         var _loc3_:String = null;
         _loc3_ = IValidatorListener(param1).errorString;
         if(!_loc3_ && Boolean(param2))
         {
            param1.addEventListener(MouseEvent.MOUSE_OVER,errorTipMouseOverHandler);
            param1.addEventListener(MouseEvent.MOUSE_OUT,errorTipMouseOutHandler);
         }
         else if(Boolean(_loc3_) && !param2)
         {
            param1.removeEventListener(MouseEvent.MOUSE_OVER,errorTipMouseOverHandler);
            param1.removeEventListener(MouseEvent.MOUSE_OUT,errorTipMouseOutHandler);
         }
      }
      
      private function initialize() : void
      {
         if(!showTimer)
         {
            showTimer = new Timer(0,1);
            showTimer.addEventListener(TimerEvent.TIMER,showTimer_timerHandler);
         }
         if(!hideTimer)
         {
            hideTimer = new Timer(0,1);
            hideTimer.addEventListener(TimerEvent.TIMER,hideTimer_timerHandler);
         }
         if(!scrubTimer)
         {
            scrubTimer = new Timer(0,1);
         }
         initialized = true;
      }
      
      public function destroyToolTip(param1:IToolTip) : void
      {
         var _loc2_:ISystemManager = null;
         _loc2_ = param1.systemManager;
         _loc2_.toolTipChildren.removeChild(DisplayObject(param1));
      }
      
      private function checkIfTargetChanged(param1:DisplayObject) : void
      {
         if(!enabled)
         {
            return;
         }
         findTarget(param1);
         if(currentTarget != previousTarget)
         {
            targetChanged();
            previousTarget = currentTarget;
         }
      }
      
      private function changeHandler(param1:Event) : void
      {
         reset();
      }
      
      public function set toolTipClass(param1:Class) : void
      {
         _toolTipClass = param1;
      }
      
      private function getGlobalBounds(param1:DisplayObject) : Rectangle
      {
         var _loc2_:Point = null;
         _loc2_ = new Point(0,0);
         _loc2_ = param1.localToGlobal(_loc2_);
         return new Rectangle(_loc2_.x,_loc2_.y,param1.width,param1.height);
      }
      
      private function positionTip() : void
      {
         var _loc1_:Number = NaN;
         var _loc2_:Number = NaN;
         var _loc3_:Number = NaN;
         var _loc4_:Number = NaN;
         var _loc5_:Rectangle = null;
         var _loc6_:Point = null;
         var _loc7_:IToolTip = null;
         var _loc8_:Number = NaN;
         var _loc9_:Number = NaN;
         var _loc10_:Number = NaN;
         var _loc11_:Number = NaN;
         _loc3_ = currentToolTip.screen.width;
         _loc4_ = currentToolTip.screen.height;
         if(isError)
         {
            _loc5_ = getGlobalBounds(currentTarget);
            _loc1_ = _loc5_.right + 4;
            _loc2_ = _loc5_.top - 1;
            if(_loc1_ + currentToolTip.width > _loc3_)
            {
               _loc8_ = NaN;
               _loc9_ = NaN;
               _loc1_ = _loc5_.left - 2;
               if(_loc1_ + currentToolTip.width + 4 > _loc3_)
               {
                  _loc8_ = _loc3_ - _loc1_ - 4;
                  _loc9_ = Number(Object(toolTipClass).maxWidth);
                  Object(toolTipClass).maxWidth = _loc8_;
                  if(currentToolTip is IStyleClient)
                  {
                     IStyleClient(currentToolTip).setStyle("borderStyle","errorTipAbove");
                  }
                  currentToolTip["text"] = currentToolTip["text"];
                  Object(toolTipClass).maxWidth = _loc9_;
               }
               else
               {
                  if(currentToolTip is IStyleClient)
                  {
                     IStyleClient(currentToolTip).setStyle("borderStyle","errorTipAbove");
                  }
                  currentToolTip["text"] = currentToolTip["text"];
               }
               if(currentToolTip.height + 2 < _loc5_.top)
               {
                  _loc2_ = _loc5_.top - (currentToolTip.height + 2);
               }
               else
               {
                  _loc2_ = _loc5_.bottom + 2;
                  if(!isNaN(_loc8_))
                  {
                     Object(toolTipClass).maxWidth = _loc8_;
                  }
                  if(currentToolTip is IStyleClient)
                  {
                     IStyleClient(currentToolTip).setStyle("borderStyle","errorTipBelow");
                  }
                  currentToolTip["text"] = currentToolTip["text"];
                  if(!isNaN(_loc9_))
                  {
                     Object(toolTipClass).maxWidth = _loc9_;
                  }
               }
            }
            sizeTip(currentToolTip);
            _loc6_ = new Point(_loc1_,_loc2_);
            _loc7_ = currentToolTip;
            _loc6_ = DisplayObject(_loc7_).root.globalToLocal(_loc6_);
            _loc1_ = _loc6_.x;
            _loc2_ = _loc6_.y;
         }
         else
         {
            _loc1_ = Application.application.mouseX + 11;
            _loc2_ = Application.application.mouseY + 22;
            _loc10_ = Number(currentToolTip.width);
            if(_loc1_ + _loc10_ > _loc3_)
            {
               _loc1_ = _loc3_ - _loc10_;
            }
            _loc11_ = Number(currentToolTip.height);
            if(_loc2_ + _loc11_ > _loc4_)
            {
               _loc2_ = _loc4_ - _loc11_;
            }
         }
         currentToolTip.move(_loc1_,_loc2_);
      }
      
      private function errorTipMouseOutHandler(param1:MouseEvent) : void
      {
         checkIfTargetChanged(param1.relatedObject);
      }
      
      private function findTarget(param1:DisplayObject) : void
      {
         while(param1)
         {
            if(param1 is IValidatorListener)
            {
               currentText = IValidatorListener(param1).errorString;
               if(currentText != null && currentText != "")
               {
                  currentTarget = param1;
                  isError = true;
                  return;
               }
            }
            if(param1 is IToolTipManagerClient)
            {
               currentText = IToolTipManagerClient(param1).toolTip;
               if(currentText != null)
               {
                  currentTarget = param1;
                  isError = false;
                  return;
               }
            }
            param1 = param1.parent;
         }
         currentText = null;
         currentTarget = null;
      }
      
      public function registerToolTip(param1:DisplayObject, param2:String) : void
      {
         var _loc3_:String = null;
         if(param1 is IToolTipManagerClient)
         {
            _loc3_ = IToolTipManagerClient(param1).toolTip;
         }
         if(!_loc3_ && Boolean(param2))
         {
            param1.addEventListener(MouseEvent.MOUSE_OVER,toolTipMouseOverHandler);
            param1.addEventListener(MouseEvent.MOUSE_OUT,toolTipMouseOutHandler);
         }
         else if(Boolean(_loc3_) && !param2)
         {
            param1.removeEventListener(MouseEvent.MOUSE_OVER,toolTipMouseOverHandler);
            param1.removeEventListener(MouseEvent.MOUSE_OUT,toolTipMouseOutHandler);
         }
      }
      
      public function set hideDelay(param1:Number) : void
      {
         _hideDelay = param1;
      }
      
      private function getSystemManager(param1:DisplayObject) : ISystemManager
      {
         return param1 is IUIComponent ? IUIComponent(param1).systemManager : null;
      }
      
      public function set currentTarget(param1:DisplayObject) : void
      {
         _currentTarget = param1;
      }
      
      public function set showEffect(param1:Effect) : void
      {
         _showEffect = param1;
      }
      
      public function sizeTip(param1:IToolTip) : void
      {
         if(param1 is IInvalidating)
         {
            IInvalidating(param1).validateNow();
         }
         param1.setActualSize(param1.getExplicitOrMeasuredWidth(),param1.getExplicitOrMeasuredHeight());
      }
      
      public function set hideEffect(param1:Effect) : void
      {
         _hideEffect = param1;
      }
      
      private function hideTimer_timerHandler(param1:TimerEvent) : void
      {
         hideTip();
      }
      
      private function targetChanged() : void
      {
         var _loc1_:ToolTipEvent = null;
         if(!initialized)
         {
            initialize();
         }
         if(Boolean(previousTarget) && Boolean(currentToolTip))
         {
            _loc1_ = new ToolTipEvent(ToolTipEvent.TOOL_TIP_HIDE);
            _loc1_.toolTip = currentToolTip;
            previousTarget.dispatchEvent(_loc1_);
         }
         reset();
         if(currentTarget)
         {
            if(currentText == "")
            {
               return;
            }
            _loc1_ = new ToolTipEvent(ToolTipEvent.TOOL_TIP_START);
            currentTarget.dispatchEvent(_loc1_);
            if(showDelay == 0 || scrubTimer.running)
            {
               createTip();
               initializeTip();
               positionTip();
               showTip();
            }
            else
            {
               showTimer.delay = showDelay;
               showTimer.start();
            }
         }
      }
      
      private function initializeTip() : void
      {
         if(currentToolTip is ToolTip)
         {
            ToolTip(currentToolTip).text = currentText;
         }
         if(isError && currentToolTip is IStyleClient)
         {
            IStyleClient(currentToolTip).setStyle("styleName","errorTip");
         }
         sizeTip(currentToolTip);
         if(currentToolTip is IStyleClient)
         {
            if(showEffect)
            {
               IStyleClient(currentToolTip).setStyle("showEffect",showEffect);
            }
            if(hideEffect)
            {
               IStyleClient(currentToolTip).setStyle("hideEffect",hideEffect);
            }
         }
         if(Boolean(showEffect) || Boolean(hideEffect))
         {
            currentToolTip.addEventListener(EffectEvent.EFFECT_END,effectEndHandler);
         }
      }
      
      public function get showEffect() : Effect
      {
         return _showEffect;
      }
      
      private function hideTip() : void
      {
         var _loc1_:ToolTipEvent = null;
         var _loc2_:ISystemManager = null;
         if(previousTarget)
         {
            _loc1_ = new ToolTipEvent(ToolTipEvent.TOOL_TIP_HIDE);
            _loc1_.toolTip = currentToolTip;
            previousTarget.dispatchEvent(_loc1_);
         }
         if(currentToolTip)
         {
            currentToolTip.visible = false;
         }
         if(isError)
         {
            if(currentTarget)
            {
               currentTarget.removeEventListener("change",changeHandler);
            }
         }
         else if(previousTarget)
         {
            _loc2_ = getSystemManager(previousTarget);
            _loc2_.removeEventListener(MouseEvent.MOUSE_DOWN,systemManager_mouseDownHandler);
         }
         if(!hideEffect)
         {
            hideEffectEnded();
         }
      }
      
      private function createTip() : void
      {
         var _loc1_:ToolTipEvent = null;
         var _loc2_:ISystemManager = null;
         _loc1_ = new ToolTipEvent(ToolTipEvent.TOOL_TIP_CREATE);
         currentTarget.dispatchEvent(_loc1_);
         if(_loc1_.toolTip)
         {
            currentToolTip = _loc1_.toolTip;
         }
         else
         {
            currentToolTip = new toolTipClass();
         }
         currentToolTip.visible = false;
         _loc2_ = getSystemManager(currentTarget);
         _loc2_.toolTipChildren.addChild(DisplayObject(currentToolTip));
      }
   }
}

